Imports System.Drawing.Drawing2D
Imports System.Runtime.InteropServices
Public Class Form1

  ' vb@rchiv - Das groe Visual Basic Archiv
  '
  ' Copyright 2000-2009 Dieter Otter
  '
  ' Der Programmcode darf fr eigene Zwecke verwendet werden.
  ' Es ist nicht erlaubt Inhalte des Projektes (Source) ohne unserer
  ' Zustimmung zum Download anzubieten.
  '
  ' Die Beispielskripte sind Computerprogramme, die gem
  ' des 2 Abs. 1 Nr. 69 aff. UrhG den urheberrechtlichen
  ' Schutz geniessen und drfen nicht fr eigene ausgegeben
  ' werden.
  '
  ' Dieter Otter
  ' Software-Entwicklung & Vertrieb
  ' info@vbarchiv.de
  ' http://www.vbarchiv.de
  '
  ' info@tools4vb.de
  ' http://www.tools4vb.de
  '======================================================

  Public Class lvColumnHeaderRect
    ' bentigte API-Funktionen, um das Handle der ColumnHeader zu ermitteln
    Private Delegate Function WinCallBack( _
      ByVal hWnd As IntPtr, _
      ByVal lParam As IntPtr) As Boolean

    <DllImport("user32.Dll")> _
    Private Shared Function EnumChildWindows(ByVal hWndParent As IntPtr, _
      ByVal callBackFunc As WinCallBack, _
      ByVal lParam As IntPtr) As Integer
    End Function

    <DllImport("user32.dll")> _
    Private Shared Function GetWindowRect(ByVal hWnd As IntPtr, _
      ByRef lpRect As RECT) As Integer
    End Function

    <StructLayout(LayoutKind.Sequential)> _
    Private Structure RECT
      Public Left As Integer
      Public Top As Integer
      Public Right As Integer
      Public Bottom As Integer
    End Structure

    Private _headerRect As Rectangle = Rectangle.Empty
    Private _columns() As ColumnHeader

    ' Unsere CallBack-Funktion zum Ermitteln des Handles der ColumnHeader-Leiste
    Private Function lvCallBack(ByVal hWnd As IntPtr, ByVal lParam As IntPtr) As Boolean
      Dim r As RECT
      If GetWindowRect(hWnd, r) <> 0 Then
        _headerRect = New Rectangle(r.Left, r.Top, r.Right - r.Left, r.Bottom - r.Top)
      End If
      Return False
    End Function

    ''' <summary>
    ''' Initialisiert die Klasse mit dem angegebenen ListView-Objekt
    ''' </summary>
    ''' <param name="ListView">ListView-Objekt</param>
    Public Sub New(ByVal ListView As ListView)
      ' Handle der ColumnHeader-Leiste ermitteln
      EnumChildWindows(ListView.Handle.ToInt32, New WinCallBack(AddressOf lvCallBack), IntPtr.Zero)

      ' ColumnHeader-Objekte in der angezeigten Reihenfolge
      ReDim _columns(ListView.Columns.Count)
      For Each column As ColumnHeader In ListView.Columns
        _columns(column.DisplayIndex) = column
      Next
    End Sub

    ''' <summary>
    ''' Gibt das Rechteck der ColumnHeader-Leiste zurck
    ''' </summary>
    Public ReadOnly Property Rectangle() As Rectangle
      Get
        Return _headerRect
      End Get
    End Property

    ''' <summary>
    ''' Ermittelt die Spalte (das ColumnHeader-Objekt) unter dem Mauszeiger
    ''' </summary>
    ''' <param name="p">Position des Mauszeigers</param>
    ''' <value></value>
    Public ReadOnly Property ColumnHeader(ByVal p As Point) As ColumnHeader
      Get
        Dim x As Integer = _headerRect.X
        For Each column As ColumnHeader In _columns
          x += column.Width
          If p.X <= x Then
            Return column
          End If
        Next
        Return Nothing
      End Get
    End Property

  End Class

  ' Hilfs-Kontextmen
  Dim WithEvents lvMenu As New ContextMenuStrip

  Private Sub imgLogo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles imgLogo.Click
    ' URL laden
    System.Diagnostics.Process.Start("http://www.vbarchiv.net")
  End Sub

  Private Sub lnkURL_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles lnkURL.Click
    ' URL laden
    System.Diagnostics.Process.Start("http://www.vbarchiv.net")
  End Sub

  Private Sub Panel2_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles Panel2.Paint
    ' Hintergrund-Farbverlauf erstellen
    ' Gre der Form
    Dim oRect As New Rectangle(0, 0, Panel2.Width, Panel2.Height)

    ' Neues Brush-Objekt mit den gewnschten Farben und
    ' Farbverlauf-Richtung erstellen
    Dim oBrush As New LinearGradientBrush(oRect, Color.FromArgb(255, 231, 247, 255), Color.FromArgb(255, 181, 214, 255), _
      LinearGradientMode.Vertical)

    ' Hintergrund-Farbverlauf anzeigen
    e.Graphics.FillRectangle(oBrush, oRect)
  End Sub

  Private Sub Panel1_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles Panel1.Paint
    ' Hintergrund-Farbverlauf erstellen
    ' Gre der Form
    Dim oRect As New Rectangle(0, 0, Panel1.Width, Panel1.Height)

    ' Neues Brush-Objekt mit den gewnschten Farben und
    ' Farbverlauf-Richtung erstellen
    Dim oBrush As New LinearGradientBrush(oRect, Color.FromArgb(255, 231, 247, 255), Color.FromArgb(255, 181, 214, 255), _
      LinearGradientMode.Vertical)

    ' Hintergrund-Farbverlauf anzeigen
    e.Graphics.FillRectangle(oBrush, oRect)
  End Sub

  ''' <summary>
  ''' Fgt dem ListView eine komplette Datenzeile hinzu
  ''' </summary>
  ''' <param name="lvw">ListView-Control</param>
  ''' <param name="Text">Parameterliste der einzelnen Zellenwerte</param>
  Public Sub lvwAddItem(ByVal lvw As ListView, ByVal ParamArray Text() As String)
    With lvw.Items
      .Add(New ListViewItem(Text))
    End With
  End Sub

  Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
    ' ListView fllen
    For i As Integer = 1 To 10
      lvwAddItem(ListView1, "Zeile " & CStr(i), "", "", "", "")
    Next

    ' Hilfs-Kontextmen dem ListView zuweisen
    ListView1.ContextMenuStrip = lvMenu
  End Sub

  ' Bei Rechtsklick will VB das Kontextmen anzeigen
  Private Sub lvMenu_Opening(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles lvMenu.Opening
    ' Anzeige des Kontextmens unterbinden
    e.Cancel = True

    ' ColumnHeader-Rechteck ermitteln
    With New lvColumnHeaderRect(ListView1)
      ' Prfen, ob sich die Maus in der Headerzeile befindet
      If .Rectangle.Contains(Control.MousePosition) Then
        ' Rechtsklick in den ColumnHeader-Bereich
        ' ggf. Spalte ermitteln
        Dim column As ColumnHeader = .ColumnHeader(Control.MousePosition)
        If Not IsNothing(column) Then
          MsgBox("Rechtsklick auf Spalte '" & column.Text & "'")
        End If
      End If
    End With
  End Sub

End Class
